@@ -34,6 +34,8 @@
 WINE_DEFAULT_DEBUG_CHANNEL(kernelbase);
 
 
+HANDLE kernel32_handle = 0;
+
 /***********************************************************************
  *           DllMain
  */
@@ -42,6 +44,7 @@ BOOL WINAPI DllMain( HINSTANCE hinst, DWORD reason, LPVOID reserved )
     if (reason == DLL_PROCESS_ATTACH)
     {
         DisableThreadLibraryCalls( hinst );
+        kernel32_handle = GetModuleHandleA( "kernel32.dll" );
         init_startup_info( NtCurrentTeb()->Peb->ProcessParameters );
     }
     return TRUE;
