@@ -1151,6 +1151,17 @@ BOOL WINAPI SetPrivateObjectSecurity( SECURITY_INFORMATION info, PSECURITY_DESCR
     return TRUE;
 }
 
+/*************************************************************************
+ * SetPrivateObjectSecurityEx    (kernelbase.@)
+ */
+BOOL WINAPI SetPrivateObjectSecurityEx( SECURITY_INFORMATION info, PSECURITY_DESCRIPTOR descr,
+                                        PSECURITY_DESCRIPTOR *obj_descr, ULONG flags,
+                                        PGENERIC_MAPPING mapping, HANDLE token )
+{
+    FIXME( "0x%08x %p %p %u %p %p - stub\n", info, descr, obj_descr, flags, mapping, token );
+    return TRUE;
+}
+
 /******************************************************************************
  * SetSecurityDescriptorControl    (kernelbase.@)
  */
