@@ -656,6 +656,71 @@ static void test_topology(void)
     IMFTopologyNode_Release(node);
     IMFTopologyNode_Release(node2);
 
+    /* Cloning nodes of different types. */
+    hr = MFCreateTopologyNode(MF_TOPOLOGY_SOURCESTREAM_NODE, &node);
+    ok(hr == S_OK, "Failed to create topology node, hr %#x.\n", hr);
+
+    hr = MFCreateTopologyNode(MF_TOPOLOGY_OUTPUT_NODE, &node2);
+    ok(hr == S_OK, "Failed to create topology node, hr %#x.\n", hr);
+
+    hr = IMFTopologyNode_CloneFrom(node, node2);
+    ok(hr == MF_E_INVALIDREQUEST, "Unexpected hr %#x.\n", hr);
+
+    IMFTopologyNode_Release(node2);
+
+    /* Cloning preferred types. */
+    hr = MFCreateTopologyNode(MF_TOPOLOGY_SOURCESTREAM_NODE, &node2);
+    ok(hr == S_OK, "Failed to create topology node, hr %#x.\n", hr);
+
+    hr = MFCreateMediaType(&mediatype);
+    ok(hr == S_OK, "Failed to create media type, hr %#x.\n", hr);
+
+    hr = IMFTopologyNode_SetOutputPrefType(node2, 0, mediatype);
+    ok(hr == S_OK, "Failed to set preferred type, hr %#x.\n", hr);
+
+    /* Vista checks for additional attributes. */
+    hr = IMFTopologyNode_CloneFrom(node, node2);
+    ok(hr == S_OK || broken(hr == MF_E_ATTRIBUTENOTFOUND) /* Vista */, "Failed to clone a node, hr %#x.\n", hr);
+
+    hr = IMFTopologyNode_GetOutputPrefType(node, 0, &mediatype2);
+    ok(hr == S_OK, "Failed to get preferred type, hr %#x.\n", hr);
+    ok(mediatype == mediatype2, "Unexpected media type.\n");
+
+    IMFMediaType_Release(mediatype2);
+    IMFMediaType_Release(mediatype);
+
+    IMFTopologyNode_Release(node2);
+
+    /* Existing preferred types are not cleared. */
+    hr = MFCreateTopologyNode(MF_TOPOLOGY_SOURCESTREAM_NODE, &node2);
+    ok(hr == S_OK, "Failed to create topology node, hr %#x.\n", hr);
+
+    hr = IMFTopologyNode_GetOutputCount(node, &count);
+    ok(hr == S_OK, "Failed to get output count, hr %#x.\n", hr);
+    ok(count == 1, "Unexpected output count.\n");
+
+    hr = IMFTopologyNode_CloneFrom(node, node2);
+    ok(hr == S_OK || broken(hr == MF_E_ATTRIBUTENOTFOUND) /* Vista */, "Failed to clone a node, hr %#x.\n", hr);
+
+    hr = IMFTopologyNode_GetOutputCount(node, &count);
+    ok(hr == S_OK, "Failed to get output count, hr %#x.\n", hr);
+    ok(count == 1, "Unexpected output count.\n");
+
+    hr = IMFTopologyNode_GetOutputPrefType(node, 0, &mediatype2);
+    ok(hr == S_OK, "Failed to get preferred type, hr %#x.\n", hr);
+    ok(!!mediatype2, "Unexpected media type.\n");
+    IMFMediaType_Release(mediatype2);
+
+    hr = IMFTopologyNode_CloneFrom(node2, node);
+    ok(hr == S_OK || broken(hr == MF_E_ATTRIBUTENOTFOUND) /* Vista */, "Failed to clone a node, hr %#x.\n", hr);
+
+    hr = IMFTopologyNode_GetOutputCount(node2, &count);
+    ok(hr == S_OK, "Failed to get output count, hr %#x.\n", hr);
+    ok(count == 1, "Unexpected output count.\n");
+
+    IMFTopologyNode_Release(node2);
+    IMFTopologyNode_Release(node);
+
     IMFTopology_Release(topology);
 }
 
@@ -785,8 +850,9 @@ static void test_MFGetService(void)
     ok(unk == (void *)0xdeadbeef, "Unexpected out object.\n");
 }
 
-static void test_MFCreateSequencerSource(void)
+static void test_sequencer_source(void)
 {
+    IMFMediaSourceTopologyProvider *provider;
     IMFSequencerSource *seq_source;
     HRESULT hr;
 
@@ -796,6 +862,10 @@ static void test_MFCreateSequencerSource(void)
     hr = MFCreateSequencerSource(NULL, &seq_source);
     ok(hr == S_OK, "Failed to create sequencer source, hr %#x.\n", hr);
 
+    hr = IMFSequencerSource_QueryInterface(seq_source, &IID_IMFMediaSourceTopologyProvider, (void **)&provider);
+    ok(hr == S_OK, "Failed to get provider interface, hr %#x.\n", hr);
+    IMFMediaSourceTopologyProvider_Release(provider);
+
     IMFSequencerSource_Release(seq_source);
 
     hr = MFShutdown();
@@ -952,6 +1022,7 @@ static void test_session_events(IMFMediaSession *session)
 static void test_media_session(void)
 {
     IMFRateControl *rate_control, *rate_control2;
+    IMFLocalMFTRegistration *local_reg;
     MFCLOCK_PROPERTIES clock_props;
     IMFRateSupport *rate_support;
     IMFAttributes *attributes;
@@ -981,6 +1052,12 @@ static void test_media_session(void)
     hr = IMFGetService_GetService(gs, &MF_RATE_CONTROL_SERVICE, &IID_IMFRateControl, (void **)&rate_control);
     ok(hr == S_OK, "Failed to get rate control interface, hr %#x.\n", hr);
 
+    hr = IMFGetService_GetService(gs, &MF_LOCAL_MFT_REGISTRATION_SERVICE, &IID_IMFLocalMFTRegistration,
+            (void **)&local_reg);
+    ok(hr == S_OK || broken(hr == E_NOINTERFACE) /* Vista */, "Failed to get registration service, hr %#x.\n", hr);
+    if (SUCCEEDED(hr))
+        IMFLocalMFTRegistration_Release(local_reg);
+
     hr = IMFRateSupport_QueryInterface(rate_support, &IID_IMFMediaSession, (void **)&unk);
     ok(hr == S_OK, "Failed to get session interface, hr %#x.\n", hr);
     ok(unk == (IUnknown *)session, "Unexpected pointer.\n");
@@ -1029,8 +1106,6 @@ todo_wine
 
     IMFGetService_Release(gs);
 
-    test_session_events(session);
-
     IMFMediaSession_Release(session);
 
     hr = MFCreateMediaSession(NULL, &session);
@@ -1065,6 +1140,14 @@ todo_wine
 
     IMFAttributes_Release(attributes);
 
+    /* Basic events behavior. */
+    hr = MFCreateMediaSession(NULL, &session);
+    ok(hr == S_OK, "Failed to create media session, hr %#x.\n", hr);
+
+    test_session_events(session);
+
+    IMFMediaSession_Release(session);
+
     hr = MFShutdown();
     ok(hr == S_OK, "Shutdown failure, hr %#x.\n", hr);
 }
@@ -1308,7 +1391,6 @@ todo_wine {
     ok(value == MF_TOPOLOGY_RESOLUTION_SUCCEEDED, "Unexpected value %#x.\n", value);
 }
     hr = IMFTopoLoader_Load(loader, full_topology, &topology2, NULL);
-todo_wine
     ok(hr == S_OK, "Failed to resolve topology, hr %#x.\n", hr);
     ok(full_topology != topology2, "Unexpected instance.\n");
 
@@ -1502,6 +1584,9 @@ static void test_presentation_clock(void)
     hr = MFCreatePresentationClock(&clock);
     ok(hr == S_OK, "Failed to create presentation clock, hr %#x.\n", hr);
 
+    hr = IMFPresentationClock_QueryInterface(clock, &IID_IMFRateControl, (void **)&rate_control);
+    ok(hr == S_OK, "Failed to get rate control interface, hr %#x.\n", hr);
+
     hr = IMFPresentationClock_GetTimeSource(clock, &time_source);
     ok(hr == MF_E_CLOCK_NO_TIME_SOURCE, "Unexpected hr %#x.\n", hr);
 
@@ -1522,7 +1607,7 @@ static void test_presentation_clock(void)
 
     value = 1;
     hr = IMFPresentationClock_GetContinuityKey(clock, &value);
-    ok(hr == S_OK, "Unexpected hr %#x.\n", hr);
+    ok(hr == S_OK, "Failed to get continuity key, hr %#x.\n", hr);
     ok(value == 0, "Unexpected value %u.\n", value);
 
     hr = IMFPresentationClock_GetProperties(clock, &props);
@@ -1560,6 +1645,19 @@ static void test_presentation_clock(void)
     hr = IMFPresentationClock_RemoveClockStateSink(clock, &test_sink);
     ok(hr == S_OK, "Unexpected hr %#x.\n", hr);
 
+    /* State change commands, time source is not set yet. */
+    hr = IMFPresentationClock_Start(clock, 0);
+    ok(hr == MF_E_CLOCK_NO_TIME_SOURCE, "Unexpected hr %#x.\n", hr);
+
+    hr = IMFPresentationClock_Pause(clock);
+    ok(hr == MF_E_CLOCK_NO_TIME_SOURCE, "Unexpected hr %#x.\n", hr);
+
+    hr = IMFPresentationClock_Stop(clock);
+    ok(hr == MF_E_CLOCK_NO_TIME_SOURCE, "Unexpected hr %#x.\n", hr);
+
+    hr = IMFRateControl_SetRate(rate_control, FALSE, 0.0f);
+    ok(hr == MF_E_CLOCK_NO_TIME_SOURCE, "Unexpected hr %#x.\n", hr);
+
     /* Set default time source. */
     hr = MFCreateSystemTimeSource(&time_source);
     ok(hr == S_OK, "Failed to create time source, hr %#x.\n", hr);
@@ -1632,9 +1730,6 @@ static void test_presentation_clock(void)
 
     IMFPresentationTimeSource_Release(time_source);
 
-    hr = IMFPresentationClock_QueryInterface(clock, &IID_IMFRateControl, (void **)&rate_control);
-    ok(hr == S_OK, "Failed to get rate control interface, hr %#x.\n", hr);
-
     hr = IMFRateControl_GetRate(rate_control, NULL, &rate);
     ok(hr == S_OK, "Failed to get clock rate, hr %#x.\n", hr);
 
@@ -2071,6 +2166,32 @@ todo_wine
     refcount = IMFActivate_Release(activate);
     ok(!refcount, "Unexpected refcount %u.\n", refcount);
 
+    /* Rateless mode with MF_SAMPLEGRABBERSINK_IGNORE_CLOCK. */
+    hr = MFCreateMediaType(&media_type);
+    ok(hr == S_OK, "Failed to create media type, hr %#x.\n", hr);
+
+    hr = IMFMediaType_SetGUID(media_type, &MF_MT_MAJOR_TYPE, &MFMediaType_Audio);
+    ok(hr == S_OK, "Failed to set attribute, hr %#x.\n", hr);
+    hr = IMFMediaType_SetGUID(media_type, &MF_MT_SUBTYPE, &MFAudioFormat_PCM);
+    ok(hr == S_OK, "Failed to set attribute, hr %#x.\n", hr);
+
+    hr = MFCreateSampleGrabberSinkActivate(media_type, &grabber_callback, &activate);
+    ok(hr == S_OK, "Failed to create grabber activate, hr %#x.\n", hr);
+
+    hr = IMFActivate_SetUINT32(activate, &MF_SAMPLEGRABBERSINK_IGNORE_CLOCK, 1);
+    ok(hr == S_OK, "Failed to set attribute, hr %#x.\n", hr);
+
+    hr = IMFActivate_ActivateObject(activate, &IID_IMFMediaSink, (void **)&sink);
+    ok(hr == S_OK, "Failed to activate object, hr %#x.\n", hr);
+
+    hr = IMFMediaSink_GetCharacteristics(sink, &flags);
+    ok(hr == S_OK, "Failed to get sink flags, hr %#x.\n", hr);
+    ok(flags & MEDIASINK_RATELESS, "Unexpected flags %#x.\n", flags);
+
+    IMFMediaSink_Release(sink);
+    IMFActivate_Release(activate);
+    IMFMediaType_Release(media_type);
+
     hr = MFShutdown();
     ok(hr == S_OK, "Failed to shut down, hr %#x.\n", hr);
 }
@@ -2431,16 +2552,49 @@ static void test_quality_manager(void)
     IMFQualityManager_Release(manager);
 }
 
+static void test_sar(void)
+{
+    IMFPresentationTimeSource *time_source;
+    IMFMediaSink *sink;
+    HRESULT hr;
+
+    hr = CoInitialize(NULL);
+    ok(hr == S_OK, "Failed to initialize, hr %#x.\n", hr);
+
+    hr = MFCreateAudioRenderer(NULL, &sink);
+    if (hr == MF_E_NO_AUDIO_PLAYBACK_DEVICE)
+    {
+        skip("No audio playback device available.\n");
+        CoUninitialize();
+        return;
+    }
+
+todo_wine
+    ok(hr == S_OK, "Failed to create renderer, hr %#x.\n", hr);
+
+if (SUCCEEDED(hr))
+{
+    hr = IMFMediaSink_QueryInterface(sink, &IID_IMFPresentationTimeSource, (void **)&time_source);
+    ok(hr == S_OK, "Failed to get time source interface, hr %#x.\n", hr);
+    IMFPresentationTimeSource_Release(time_source);
+
+    IMFMediaSink_Release(sink);
+}
+
+    CoUninitialize();
+}
+
 START_TEST(mf)
 {
     test_topology();
     test_topology_loader();
     test_MFGetService();
-    test_MFCreateSequencerSource();
+    test_sequencer_source();
     test_media_session();
     test_MFShutdownObject();
     test_presentation_clock();
     test_sample_grabber();
     test_video_processor();
     test_quality_manager();
+    test_sar();
 }
