@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-
 #include <stdarg.h>
 #include <assert.h>
 
@@ -76,7 +74,7 @@ static PRUnichar *handle_insert_comment(HTMLDocumentNode *doc, const PRUnichar *
         return NULL;
 
     ptr = comment+3;
-    while(isspaceW(*ptr))
+    while(iswspace(*ptr))
         ptr++;
 
     if(ptr[0] == 'l' && ptr[1] == 't') {
@@ -97,40 +95,40 @@ static PRUnichar *handle_insert_comment(HTMLDocumentNode *doc, const PRUnichar *
         }
     }
 
-    if(!isspaceW(*ptr++))
+    if(!iswspace(*ptr++))
         return NULL;
-    while(isspaceW(*ptr))
+    while(iswspace(*ptr))
         ptr++;
 
     if(ptr[0] != 'I' || ptr[1] != 'E')
         return NULL;
 
     ptr +=2;
-    if(!isspaceW(*ptr++))
+    if(!iswspace(*ptr++))
         return NULL;
-    while(isspaceW(*ptr))
+    while(iswspace(*ptr))
         ptr++;
 
-    if(!isdigitW(*ptr))
+    if(!iswdigit(*ptr))
         return NULL;
-    while(isdigitW(*ptr))
+    while(iswdigit(*ptr))
         majorv = majorv*10 + (*ptr++ - '0');
 
     if(*ptr == '.') {
         ptr++;
-        if(!isdigitW(*ptr))
+        if(!iswdigit(*ptr))
             return NULL;
-        while(isdigitW(*ptr))
+        while(iswdigit(*ptr))
             minorv = minorv*10 + (*ptr++ - '0');
     }
 
-    while(isspaceW(*ptr))
+    while(iswspace(*ptr))
         ptr++;
     if(ptr[0] != ']' || ptr[1] != '>')
         return NULL;
     ptr += 2;
 
-    len = strlenW(ptr);
+    len = lstrlenW(ptr);
     if(len < ARRAY_SIZE(endifW))
         return NULL;
 
@@ -407,7 +405,7 @@ BOOL parse_compat_version(const WCHAR *version_string, compat_mode_t *r)
 
     for(p = version_string; '0' <= *p && *p <= '9'; p++)
         version = version * 10 + *p-'0';
-    if(*p || p == version_string)
+    if((*p && *p != ';') || p == version_string)
         return FALSE;
 
     switch(version){
@@ -436,15 +434,18 @@ BOOL parse_compat_version(const WCHAR *version_string, compat_mode_t *r)
 static BOOL parse_ua_compatible(const WCHAR *p, compat_mode_t *r)
 {
     static const WCHAR ie_eqW[] = {'I','E','='};
-    static const WCHAR edgeW[] = {'e','d','g','e',0};
+    static const WCHAR edgeW[] = {'e','d','g','e'};
 
     TRACE("%s\n", debugstr_w(p));
 
-    if(strncmpiW(ie_eqW, p, ARRAY_SIZE(ie_eqW)))
+    if(wcsnicmp(ie_eqW, p, ARRAY_SIZE(ie_eqW)))
         return FALSE;
     p += 3;
 
-    if(!strcmpiW(p, edgeW)) {
+    if(!wcsnicmp(p, edgeW, ARRAY_SIZE(edgeW))) {
+        p += ARRAY_SIZE(edgeW);
+        if(*p && *p != ';')
+            return FALSE;
         *r = COMPAT_MODE_IE11;
         return TRUE;
     }
@@ -506,7 +507,7 @@ static void process_meta_element(HTMLDocumentNode *doc, nsIDOMHTMLMetaElement *m
 
         TRACE("%s: %s\n", debugstr_w(http_equiv), debugstr_w(content));
 
-        if(!strcmpiW(http_equiv, x_ua_compatibleW)) {
+        if(!wcsicmp(http_equiv, x_ua_compatibleW)) {
             compat_mode_t document_mode;
             if(parse_ua_compatible(content, &document_mode))
                 set_document_mode(doc, document_mode, TRUE);
