@@ -52,13 +52,13 @@ static int FUNC_NAME(puts_clbk_str)(void *ctx, int len, const APICHAR *str)
         return len;
 
     if(out->len < len) {
-        memcpy(out->buf, str, out->len*sizeof(APICHAR));
+        memmove(out->buf, str, out->len*sizeof(APICHAR));
         out->buf += out->len;
         out->len = 0;
         return -1;
     }
 
-    memcpy(out->buf, str, len*sizeof(APICHAR));
+    memmove(out->buf, str, len*sizeof(APICHAR));
     out->buf += len;
     out->len -= len;
     return len;
@@ -354,7 +354,7 @@ static inline void FUNC_NAME(pf_fixup_exponent)(char *buf, BOOL three_digit_exp)
 {
     char* tmp = buf;
 
-    while(tmp[0] && toupper(tmp[0])!='E')
+    while(tmp[0] && MSVCRT__toupper_l(tmp[0], NULL)!='E')
         tmp++;
 
     if(tmp[0] && (tmp[1]=='+' || tmp[1]=='-') &&
@@ -516,10 +516,14 @@ int FUNC_NAME(pf_printf)(FUNC_NAME(puts_clbk) pf_puts, void *puts_ctx, const API
             } else if(*p == 'w')
                 flags.WideString = *p++;
 #if _MSVCR_VER >= 140
-            else if(*p == 'z')
+            else if(*p == 'z' || *p == 't')
                 flags.IntegerNative = *p++;
             else if(*p == 'T')
                 flags.NaturalString = *p++;
+            else if(*p == 'j') {
+                flags.IntegerDouble++;
+                p++;
+            }
 #endif
             else if((*p == 'F' || *p == 'N') && legacy_msvcrt_compat)
                 p++; /* ignore */
@@ -664,10 +668,10 @@ int FUNC_NAME(pf_printf)(FUNC_NAME(puts_clbk) pf_puts, void *puts_ctx, const API
                     sprintf(tmp, nan_str);
                 if (strchr("EFG", flags.Format))
                     for(i=0; tmp[i]; i++)
-                        tmp[i] = toupper(tmp[i]);
+                        tmp[i] = MSVCRT__toupper_l(tmp[i], NULL);
             } else {
                 sprintf(tmp, float_fmt, val);
-                if(toupper(flags.Format)=='E' || toupper(flags.Format)=='G')
+                if(MSVCRT__toupper_l(flags.Format, NULL)=='E' || MSVCRT__toupper_l(flags.Format, NULL)=='G')
                     FUNC_NAME(pf_fixup_exponent)(tmp, three_digit_exp);
             }
 
