@@ -107,6 +107,7 @@ typedef struct MSVCRT__exception {
 typedef int (CDECL *MSVCRT_matherr_func)(struct MSVCRT__exception *);
 
 static HMODULE module;
+static LONGLONG crt_init_start, crt_init_end;
 
 static int (CDECL *p_initialize_onexit_table)(MSVCRT__onexit_table_t *table);
 static int (CDECL *p_register_onexit_function)(MSVCRT__onexit_table_t *table, MSVCRT__onexit_t func);
@@ -137,6 +138,7 @@ static void (CDECL *p___setusermatherr)(MSVCRT_matherr_func);
 static int* (CDECL *p_errno)(void);
 static char* (CDECL *p_asctime)(const struct tm *);
 static size_t (__cdecl *p_strftime)(char *, size_t, const char *, const struct tm *);
+static struct tm*  (__cdecl *p__gmtime32)(const __time32_t*);
 static void (CDECL *p_exit)(int);
 static int (CDECL *p__crt_atexit)(void (CDECL*)(void));
 static int (__cdecl *p_crt_at_quick_exit)(void (__cdecl *func)(void));
@@ -146,6 +148,7 @@ static int (__cdecl *p__close)(int);
 static void* (__cdecl *p__o_malloc)(size_t);
 static size_t (__cdecl *p__msize)(void*);
 static void (__cdecl *p_free)(void*);
+static clock_t (__cdecl *p_clock)(void);
 
 static void test__initialize_onexit_table(void)
 {
@@ -361,11 +364,11 @@ static void test___fpe_flt_rounds(void)
 
     ok((p__controlfp(_RC_UP, _RC_CHOP) & _RC_CHOP) == _RC_UP, "_controlfp(_RC_UP, _RC_CHOP) failed\n");
     ret = p___fpe_flt_rounds();
-    ok(ret == 2 + (sizeof(void*)>sizeof(int)), "__fpe_flt_rounds returned %d\n", ret);
+    ok(ret == 2 || broken(ret == 3) /* w1064v1507 */, "__fpe_flt_rounds returned %d\n", ret);
 
     ok((p__controlfp(_RC_DOWN, _RC_CHOP) & _RC_CHOP) == _RC_DOWN, "_controlfp(_RC_DOWN, _RC_CHOP) failed\n");
     ret = p___fpe_flt_rounds();
-    ok(ret == 3 - (sizeof(void*)>sizeof(int)), "__fpe_flt_rounds returned %d\n", ret);
+    ok(ret == 3 || broken(ret == 2) /* w1064v1507 */, "__fpe_flt_rounds returned %d\n", ret);
 
     ok((p__controlfp(_RC_CHOP, _RC_CHOP) & _RC_CHOP) == _RC_CHOP, "_controlfp(_RC_CHOP, _RC_CHOP) failed\n");
     ret = p___fpe_flt_rounds();
@@ -465,7 +468,13 @@ static void test__get_narrow_winmain_command_line(char *path)
 
 static BOOL init(void)
 {
+    FILETIME cur;
+
+    GetSystemTimeAsFileTime(&cur);
+    crt_init_start = ((LONGLONG)cur.dwHighDateTime << 32) + cur.dwLowDateTime;
     module = LoadLibraryA("ucrtbase.dll");
+    GetSystemTimeAsFileTime(&cur);
+    crt_init_end = ((LONGLONG)cur.dwHighDateTime << 32) + cur.dwLowDateTime;
 
     if(!module) {
         win_skip("ucrtbase.dll not available\n");
@@ -501,6 +510,7 @@ static BOOL init(void)
     p_errno = (void*)GetProcAddress(module, "_errno");
     p_asctime = (void*)GetProcAddress(module, "asctime");
     p_strftime = (void*)GetProcAddress(module, "strftime");
+    p__gmtime32 = (void*)GetProcAddress(module, "_gmtime32");
     p__crt_atexit = (void*)GetProcAddress(module, "_crt_atexit");
     p_exit = (void*)GetProcAddress(module, "exit");
     p_crt_at_quick_exit = (void*)GetProcAddress(module, "_crt_at_quick_exit");
@@ -510,6 +520,7 @@ static BOOL init(void)
     p__o_malloc = (void*)GetProcAddress(module, "_o_malloc");
     p__msize = (void*)GetProcAddress(module, "_msize");
     p_free = (void*)GetProcAddress(module, "free");
+    p_clock = (void*)GetProcAddress(module, "clock");
 
     return TRUE;
 }
@@ -892,8 +903,47 @@ static void test_asctime(void)
 static void test_strftime(void)
 {
     const struct tm epoch = { 0, 0, 0, 1, 0, 70, 4, 0, 0 };
+    const struct tm tm1 = { 0, 0, 0, 1, 0, 117, 0, 0, 0 };
+    const struct tm tm2 = { 0, 0, 14, 1, 0, 121, 6, 0, 0 };
     char bufA[256];
     size_t retA;
+    int i;
+
+    retA = p_strftime(bufA, sizeof(bufA), "%C", &epoch);
+    ok(retA == 2, "expected 2, got %d\n", (int)retA);
+    ok(!strcmp(bufA, "19"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%D", &epoch);
+    ok(retA == 8, "expected 8, got %d\n", (int)retA);
+    ok(!strcmp(bufA, "01/01/70"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%#D", &epoch);
+    ok(retA == 6, "expected 6, got %d\n", (int)retA);
+    ok(!strcmp(bufA, "1/1/70"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%e", &epoch);
+    ok(retA == 2, "expected 2, got %d\n", (int)retA);
+    ok(!strcmp(bufA, " 1"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%#e", &epoch);
+    ok(retA == 1, "expected 1, got %d\n", (int)retA);
+    ok(!strcmp(bufA, "1"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%F", &epoch);
+    ok(retA == 10, "expected 10, got %d\n", (int)retA);
+    ok(!strcmp(bufA, "1970-01-01"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%#F", &epoch);
+    ok(retA == 8, "expected 8, got %d\n", (int)retA);
+    ok(!strcmp(bufA, "1970-1-1"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%R", &epoch);
+    ok(retA == 5, "expected 5, got %d\n", (int)retA);
+    ok(!strcmp(bufA, "00:00"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%#R", &epoch);
+    ok(retA == 3, "expected 3, got %d\n", (int)retA);
+    ok(!strcmp(bufA, "0:0"), "got %s\n", bufA);
 
     retA = p_strftime(bufA, sizeof(bufA), "%T", &epoch);
     ok(retA == 8, "expected 8, got %d\n", (int)retA);
@@ -902,6 +952,78 @@ static void test_strftime(void)
     retA = p_strftime(bufA, sizeof(bufA), "%#T", &epoch);
     ok(retA == 5, "expected 5, got %d\n", (int)retA);
     ok(!strcmp(bufA, "0:0:0"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%u", &tm1);
+    ok(retA == 1, "expected 1, got %d\n", (int)retA);
+    ok(!strcmp(bufA, "7"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%h", &epoch);
+    ok(retA == 3, "expected 3, got %d\n", (int)retA);
+    ok(!strcmp(bufA, "Jan"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%n", &epoch);
+    ok(retA == 1, "expected 1, got %d\n", (int)retA);
+    ok(!strcmp(bufA, "\n"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%r", &epoch);
+    todo_wine ok(retA == 11, "expected 11, got %d\n", (int)retA);
+    todo_wine ok(!strcmp(bufA, "12:00:00 AM"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%r", &tm2);
+    todo_wine ok(retA == 11, "expected 11, got %d\n", (int)retA);
+    todo_wine ok(!strcmp(bufA, "02:00:00 PM"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%t", &epoch);
+    ok(retA == 1, "expected 1, got %d\n", (int)retA);
+    ok(!strcmp(bufA, "\t"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%g", &epoch);
+    ok(retA == 2, "expected 2, got %d\n", (int)retA);
+    ok(!strcmp(bufA, "70"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%g", &tm1);
+    ok(retA == 2, "expected 2, got %d\n", (int)retA);
+    ok(!strcmp(bufA, "16"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%G", &epoch);
+    ok(retA == 4, "expected 4, got %d\n", (int)retA);
+    ok(!strcmp(bufA, "1970"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%G", &tm1);
+    ok(retA == 4, "expected 4, got %d\n", (int)retA);
+    ok(!strcmp(bufA, "2016"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%V", &epoch);
+    todo_wine ok(retA == 2, "expected 2, got %d\n", (int)retA);
+    todo_wine ok(!strcmp(bufA, "01"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%V", &tm1);
+    todo_wine ok(retA == 2, "expected 2, got %d\n", (int)retA);
+    todo_wine ok(!strcmp(bufA, "52"), "got %s\n", bufA);
+
+    retA = p_strftime(bufA, sizeof(bufA), "%V", &tm2);
+    todo_wine ok(retA == 2, "expected 2, got %d\n", (int)retA);
+    todo_wine ok(!strcmp(bufA, "53"), "got %s\n", bufA);
+
+    for(i=0; i<14; i++)
+    {
+        __time32_t t = (365*2 + i - 7) * 24 * 60 * 60;
+        struct tm tm = *p__gmtime32(&t);
+
+        retA = p_strftime(bufA, sizeof(bufA), "%g", &tm);
+        ok(retA == 2, "%d) retA = %d\n", i, (int)retA);
+        if (i <= 8)
+            ok(!strcmp(bufA, "71"), "%d) got %s, expected 71\n", i, bufA);
+        else
+            ok(!strcmp(bufA, "72"), "%d) got %s, expected 72\n", i, bufA);
+
+        retA = p_strftime(bufA, sizeof(bufA), "%G", &tm);
+        ok(retA == 4, "%d) retA = %d\n", i, (int)retA);
+        if (i <= 8)
+            ok(!strcmp(bufA, "1971"), "%d) got %s, expected 1971\n", i, bufA);
+        else
+            ok(!strcmp(bufA, "1972"), "%d) got %s, expected 1972\n", i, bufA);
+    }
 }
 
 static LONG* get_failures_counter(HANDLE *map)
@@ -1118,6 +1240,21 @@ static void test__o_malloc(void)
     p_free(m);
 }
 
+static void test_clock(void)
+{
+    static const int thresh = 100;
+    int c, expect_min, expect_max;
+    FILETIME cur;
+
+    GetSystemTimeAsFileTime(&cur);
+    c = p_clock();
+
+    expect_min = (((LONGLONG)cur.dwHighDateTime << 32) + cur.dwLowDateTime - crt_init_end) / 10000;
+    expect_max = (((LONGLONG)cur.dwHighDateTime << 32) + cur.dwLowDateTime - crt_init_start) / 10000;
+    ok(c > expect_min-thresh && c < expect_max+thresh, "clock() = %d, expected range [%d, %d]\n",
+            c, expect_min, expect_max);
+}
+
 START_TEST(misc)
 {
     int arg_c;
@@ -1154,4 +1291,5 @@ START_TEST(misc)
     test_quick_exit(arg_v[0]);
     test__stat32();
     test__o_malloc();
+    test_clock();
 }
