@@ -23,6 +23,8 @@
 
 #include <stdio.h>
 #include <errno.h>
+#include <math.h>
+#include <inttypes.h>
 
 #include "windef.h"
 #include "winbase.h"
@@ -66,20 +68,6 @@ DEFINE_EXPECT(invalid_parameter_handler);
 #define UCRTBASE_PRINTF_LEGACY_MSVCRT_COMPATIBILITY      (0x0008)
 #define UCRTBASE_PRINTF_LEGACY_THREE_DIGIT_EXPONENTS     (0x0010)
 
-static inline float __port_infinity(void)
-{
-    static const unsigned __inf_bytes = 0x7f800000;
-    return *(const float *)&__inf_bytes;
-}
-#define INFINITY __port_infinity()
-
-static inline float __port_nan(void)
-{
-    static const unsigned __nan_bytes = 0x7fc00000;
-    return *(const float *)&__nan_bytes;
-}
-#define NAN __port_nan()
-
 static inline float __port_ind(void)
 {
     static const unsigned __ind_bytes = 0xffc00000;
@@ -650,24 +638,35 @@ static void test_printf_legacy_three_digit_exp(void)
 
 static void test_printf_c99(void)
 {
-    char buf[20];
+    char buf[30];
 
     /* The msvcrt compatibility flag doesn't affect whether 'z' is interpreted
      * as size_t size for integers. */
-    if (sizeof(void*) == 8) {
-        vsprintf_wrapper(0, buf, sizeof(buf), "%zx %d",
-                         (size_t) 0x12345678123456, 1);
-        ok(!strcmp(buf, "12345678123456 1"), "buf = %s\n", buf);
-        vsprintf_wrapper(UCRTBASE_PRINTF_LEGACY_MSVCRT_COMPATIBILITY,
-                         buf, sizeof(buf), "%zx %d", (size_t) 0x12345678123456, 1);
-        ok(!strcmp(buf, "12345678123456 1"), "buf = %s\n", buf);
-    } else {
-        vsprintf_wrapper(0, buf, sizeof(buf), "%zx %d",
-                         (size_t) 0x123456, 1);
-        ok(!strcmp(buf, "123456 1"), "buf = %s\n", buf);
-        vsprintf_wrapper(UCRTBASE_PRINTF_LEGACY_MSVCRT_COMPATIBILITY,
-                         buf, sizeof(buf), "%zx %d", (size_t) 0x123456, 1);
-        ok(!strcmp(buf, "123456 1"), "buf = %s\n", buf);
+     for (int i = 0; i < 2; i++) {
+        unsigned __int64 options = (i == 0) ? 0 :
+            UCRTBASE_PRINTF_LEGACY_MSVCRT_COMPATIBILITY;
+
+        /* z modifier accepts size_t argument */
+        vsprintf_wrapper(options, buf, sizeof(buf), "%zx %d", SIZE_MAX, 1);
+        if (sizeof(size_t) == 8)
+            ok(!strcmp(buf, "ffffffffffffffff 1"), "buf = %s\n", buf);
+        else
+            ok(!strcmp(buf, "ffffffff 1"), "buf = %s\n", buf);
+
+        /* j modifier with signed format accepts intmax_t argument */
+        vsprintf_wrapper(options, buf, sizeof(buf), "%jd %d", INTMAX_MIN, 1);
+        ok(!strcmp(buf, "-9223372036854775808 1"), "buf = %s\n", buf);
+
+        /* j modifier with unsigned format accepts uintmax_t argument */
+        vsprintf_wrapper(options, buf, sizeof(buf), "%ju %d", UINTMAX_MAX, 1);
+        ok(!strcmp(buf, "18446744073709551615 1"), "buf = %s\n", buf);
+
+        /* t modifier accepts ptrdiff_t argument */
+        vsprintf_wrapper(options, buf, sizeof(buf), "%td %d", PTRDIFF_MIN, 1);
+        if (sizeof(ptrdiff_t) == 8)
+            ok(!strcmp(buf, "-9223372036854775808 1"), "buf = %s\n", buf);
+        else
+            ok(!strcmp(buf, "-2147483648 1"), "buf = %s\n", buf);
     }
 }
 
